@@ -9,7 +9,7 @@
  * The copyright holder will however retain the copyright.
  * In addition to vpnd this code is put in the 'public domain' (*)
  * for all GPL/LGPL based open source software products.
- * For all other software products please contact astmail@yahoo.com
+ * For all other software products please contact ast@domdv.de
  * for a license agreement. There is no guarantee for the fitness
  * and usability of this code for any purpose. The author takes no
  * responsibility for any damages caused by the use of this code.
@@ -340,13 +340,12 @@ static WORD32 pi[BLOWFISH_SCHEDULESIZE]=
  */
 
 void CRYPTOCALL blowfish_key_schedule(WORD08 *key,WORD32 length,
-	WORD32 *schedule)
+	register WORD32 *schedule)
 {
 	register WORD32 l;	/* byte to long conversion and Xl of Blowfish */
 	register WORD32 r;	/* index and Xr of Blowfish		      */
+	register WORD32 t;	/* work buffer				      */
 	register WORD32 *p;	/* Pn of Blowfish			      */
-	register WORD32 *s;	/* Sm,n of Blowfish			      */
-	WORD32 *x;		/* roving pointer for schedule creation	      */
 	WORD32 i;		/* counter and index			      */
 
 
@@ -356,7 +355,7 @@ void CRYPTOCALL blowfish_key_schedule(WORD08 *key,WORD32 length,
 
 	/* initialization of Pn from key bytes */
 
-	for(i=0,r=0,p=schedule;i<18;i++)
+	for(i=18,r=0,p=schedule;i;i--)
 	{
 		l=key[r++];
 		if(r==length)r=0;
@@ -374,90 +373,115 @@ void CRYPTOCALL blowfish_key_schedule(WORD08 *key,WORD32 length,
 
 	/* 521 encryptions for final schedule initialization */
 
-	for(i=l=r=0,x=schedule,s=schedule+18;i<521;i++)
+	for(i=521,l=r=0,p=schedule;i;i--)
 	{
-		/* prepare encryption */
+		/* process P1 to P18 */
 
-		p=schedule;
-
-		/* do unwrapped rounds 1 to 16 */
-
-		l^=*p++;
-		r^=((s[(l>>24)&0xff]
-		  +s[((l>>16)&0xff)|0x100])^s[((l>>8)&0xff)|0x200])
-		  +s[(l&0xff)|0x300];
-		r^=*p++;
-		l^=((s[(r>>24)&0xff]
-		  +s[((r>>16)&0xff)|0x100])^s[((r>>8)&0xff)|0x200])
-		  +s[(r&0xff)|0x300];
-		l^=*p++;
-		r^=((s[(l>>24)&0xff]
-		  +s[((l>>16)&0xff)|0x100])^s[((l>>8)&0xff)|0x200])
-		  +s[(l&0xff)|0x300];
-		r^=*p++;
-		l^=((s[(r>>24)&0xff]
-		  +s[((r>>16)&0xff)|0x100])^s[((r>>8)&0xff)|0x200])
-		  +s[(r&0xff)|0x300];
-		l^=*p++;
-		r^=((s[(l>>24)&0xff]
-		  +s[((l>>16)&0xff)|0x100])^s[((l>>8)&0xff)|0x200])
-		  +s[(l&0xff)|0x300];
-		r^=*p++;
-		l^=((s[(r>>24)&0xff]
-		  +s[((r>>16)&0xff)|0x100])^s[((r>>8)&0xff)|0x200])
-		  +s[(r&0xff)|0x300];
-		l^=*p++;
-		r^=((s[(l>>24)&0xff]
-		  +s[((l>>16)&0xff)|0x100])^s[((l>>8)&0xff)|0x200])
-		  +s[(l&0xff)|0x300];
-		r^=*p++;
-		l^=((s[(r>>24)&0xff]
-		  +s[((r>>16)&0xff)|0x100])^s[((r>>8)&0xff)|0x200])
-		  +s[(r&0xff)|0x300];
-		l^=*p++;
-		r^=((s[(l>>24)&0xff]
-		  +s[((l>>16)&0xff)|0x100])^s[((l>>8)&0xff)|0x200])
-		  +s[(l&0xff)|0x300];
-		r^=*p++;
-		l^=((s[(r>>24)&0xff]
-		  +s[((r>>16)&0xff)|0x100])^s[((r>>8)&0xff)|0x200])
-		  +s[(r&0xff)|0x300];
-		l^=*p++;
-		r^=((s[(l>>24)&0xff]
-		  +s[((l>>16)&0xff)|0x100])^s[((l>>8)&0xff)|0x200])
-		  +s[(l&0xff)|0x300];
-		r^=*p++;
-		l^=((s[(r>>24)&0xff]
-		  +s[((r>>16)&0xff)|0x100])^s[((r>>8)&0xff)|0x200])
-		  +s[(r&0xff)|0x300];
-		l^=*p++;
-		r^=((s[(l>>24)&0xff]
-		  +s[((l>>16)&0xff)|0x100])^s[((l>>8)&0xff)|0x200])
-		  +s[(l&0xff)|0x300];
-		r^=*p++;
-		l^=((s[(r>>24)&0xff]
-		  +s[((r>>16)&0xff)|0x100])^s[((r>>8)&0xff)|0x200])
-		  +s[(r&0xff)|0x300];
-		l^=*p++;
-		r^=((s[(l>>24)&0xff]
-		  +s[((l>>16)&0xff)|0x100])^s[((l>>8)&0xff)|0x200])
-		  +s[(l&0xff)|0x300];
-		r^=*p++;
-		l^=((s[(r>>24)&0xff]
-		  +s[((r>>16)&0xff)|0x100])^s[((r>>8)&0xff)|0x200])
-		  +s[(r&0xff)|0x300];
-
-		/* do final Ps */
-
-		l^=*p++;
-		r^=*p;
+		l^=schedule[0];
+		t=schedule[(BITS32(l)>>24)+0x12];
+		t+=schedule[((l>>16)&0xff)+0x112];
+		t^=schedule[((l>>8)&0xff)+0x212];
+		t+=schedule[(l&0xff)+0x312];
+		r^=t;
+		r^=schedule[1];
+		t=schedule[(BITS32(r)>>24)+0x12];
+		t+=schedule[((r>>16)&0xff)+0x112];
+		t^=schedule[((r>>8)&0xff)+0x212];
+		t+=schedule[(r&0xff)+0x312];
+		l^=t;
+		l^=schedule[2];
+		t=schedule[(BITS32(l)>>24)+0x12];
+		t+=schedule[((l>>16)&0xff)+0x112];
+		t^=schedule[((l>>8)&0xff)+0x212];
+		t+=schedule[(l&0xff)+0x312];
+		r^=t;
+		r^=schedule[3];
+		t=schedule[(BITS32(r)>>24)+0x12];
+		t+=schedule[((r>>16)&0xff)+0x112];
+		t^=schedule[((r>>8)&0xff)+0x212];
+		t+=schedule[(r&0xff)+0x312];
+		l^=t;
+		l^=schedule[4];
+		t=schedule[(BITS32(l)>>24)+0x12];
+		t+=schedule[((l>>16)&0xff)+0x112];
+		t^=schedule[((l>>8)&0xff)+0x212];
+		t+=schedule[(l&0xff)+0x312];
+		r^=t;
+		r^=schedule[5];
+		t=schedule[(BITS32(r)>>24)+0x12];
+		t+=schedule[((r>>16)&0xff)+0x112];
+		t^=schedule[((r>>8)&0xff)+0x212];
+		t+=schedule[(r&0xff)+0x312];
+		l^=t;
+		l^=schedule[6];
+		t=schedule[(BITS32(l)>>24)+0x12];
+		t+=schedule[((l>>16)&0xff)+0x112];
+		t^=schedule[((l>>8)&0xff)+0x212];
+		t+=schedule[(l&0xff)+0x312];
+		r^=t;
+		r^=schedule[7];
+		t=schedule[(BITS32(r)>>24)+0x12];
+		t+=schedule[((r>>16)&0xff)+0x112];
+		t^=schedule[((r>>8)&0xff)+0x212];
+		t+=schedule[(r&0xff)+0x312];
+		l^=t;
+		l^=schedule[8];
+		t=schedule[(BITS32(l)>>24)+0x12];
+		t+=schedule[((l>>16)&0xff)+0x112];
+		t^=schedule[((l>>8)&0xff)+0x212];
+		t+=schedule[(l&0xff)+0x312];
+		r^=t;
+		r^=schedule[9];
+		t=schedule[(BITS32(r)>>24)+0x12];
+		t+=schedule[((r>>16)&0xff)+0x112];
+		t^=schedule[((r>>8)&0xff)+0x212];
+		t+=schedule[(r&0xff)+0x312];
+		l^=t;
+		l^=schedule[10];
+		t=schedule[(BITS32(l)>>24)+0x12];
+		t+=schedule[((l>>16)&0xff)+0x112];
+		t^=schedule[((l>>8)&0xff)+0x212];
+		t+=schedule[(l&0xff)+0x312];
+		r^=t;
+		r^=schedule[11];
+		t=schedule[(BITS32(r)>>24)+0x12];
+		t+=schedule[((r>>16)&0xff)+0x112];
+		t^=schedule[((r>>8)&0xff)+0x212];
+		t+=schedule[(r&0xff)+0x312];
+		l^=t;
+		l^=schedule[12];
+		t=schedule[(BITS32(l)>>24)+0x12];
+		t+=schedule[((l>>16)&0xff)+0x112];
+		t^=schedule[((l>>8)&0xff)+0x212];
+		t+=schedule[(l&0xff)+0x312];
+		r^=t;
+		r^=schedule[13];
+		t=schedule[(BITS32(r)>>24)+0x12];
+		t+=schedule[((r>>16)&0xff)+0x112];
+		t^=schedule[((r>>8)&0xff)+0x212];
+		t+=schedule[(r&0xff)+0x312];
+		l^=t;
+		l^=schedule[14];
+		t=schedule[(BITS32(l)>>24)+0x12];
+		t+=schedule[((l>>16)&0xff)+0x112];
+		t^=schedule[((l>>8)&0xff)+0x212];
+		t+=schedule[(l&0xff)+0x312];
+		r^=t;
+		r^=schedule[15];
+		t=schedule[(BITS32(r)>>24)+0x12];
+		t+=schedule[((r>>16)&0xff)+0x112];
+		t^=schedule[((r>>8)&0xff)+0x212];
+		t+=schedule[(r&0xff)+0x312];
+		l^=t;
+		l^=schedule[16];
+		r^=schedule[17];
 
 		/* write result to schedule and swap by the way l and r */
 
-		*x=BITS32(r);
+		*p=BITS32(r);
 		r=BITS32(l);
-		l=*x++;
-		*x++=r;
+		l=*p++;
+		*p++=r;
 	}
 }
 
@@ -476,126 +500,147 @@ void CRYPTOCALL blowfish_key_schedule(WORD08 *key,WORD32 length,
  * Blowfish algorithm.
  */
 
-void CRYPTOCALL blowfish_encrypt(WORD08 *data,WORD32 *schedule)
+void CRYPTOCALL blowfish_encrypt(WORD08 *data,register WORD32 *schedule)
 {
 	register WORD32 l;			/* Xl of Blowfish	*/
 	register WORD32 r;			/* Xr of Blowfish	*/
-	register union
-	{
-		WORD08 *d;			/* 8 bit rover		*/
-		WORD32 *p;			/* 32 bit rover		*/
-	} u;					/* rover union		*/
-	register WORD32 *s=schedule+18;		/* pointer to Sm,n	*/
+	register WORD32 t;			/* work buffer		*/
 
 
-	/* get fast pointer to data */
-
-	u.d=data;
-
 	/* load first 32 bits */
 
-	l=*u.d++;
+	l=data[0];
 	l<<=8;
-	l|=*u.d++;
+	l+=data[1];
 	l<<=8;
-	l|=*u.d++;
+	l+=data[2];
 	l<<=8;
-	l|=*u.d++;
+	l|=data[3];
 
 	/* load second 32 bits */
 
-	r=*u.d++;
+	r=data[4];
 	r<<=8;
-	r|=*u.d++;
+	r+=data[5];
 	r<<=8;
-	r|=*u.d++;
+	r+=data[6];
 	r<<=8;
-	r|=*u.d;
+	r+=data[7];
 
-	/* save current data pointer, get pointer to P1 */
+	/* process P1 to P18 */
 
-	data=u.d;
-	u.p=schedule;
-
-	/* process P1 to P16 */
-
-	l^=*u.p++;
-	r^=((s[(l>>24)&0xff]
-	  +s[((l>>16)&0xff)|0x100])^s[((l>>8)&0xff)|0x200])+s[(l&0xff)|0x300];
-	r^=*u.p++;
-	l^=((s[(r>>24)&0xff]
-	  +s[((r>>16)&0xff)|0x100])^s[((r>>8)&0xff)|0x200])+s[(r&0xff)|0x300];
-	l^=*u.p++;
-	r^=((s[(l>>24)&0xff]
-	  +s[((l>>16)&0xff)|0x100])^s[((l>>8)&0xff)|0x200])+s[(l&0xff)|0x300];
-	r^=*u.p++;
-	l^=((s[(r>>24)&0xff]
-	  +s[((r>>16)&0xff)|0x100])^s[((r>>8)&0xff)|0x200])+s[(r&0xff)|0x300];
-	l^=*u.p++;
-	r^=((s[(l>>24)&0xff]
-	  +s[((l>>16)&0xff)|0x100])^s[((l>>8)&0xff)|0x200])+s[(l&0xff)|0x300];
-	r^=*u.p++;
-	l^=((s[(r>>24)&0xff]
-	  +s[((r>>16)&0xff)|0x100])^s[((r>>8)&0xff)|0x200])+s[(r&0xff)|0x300];
-	l^=*u.p++;
-	r^=((s[(l>>24)&0xff]
-	  +s[((l>>16)&0xff)|0x100])^s[((l>>8)&0xff)|0x200])+s[(l&0xff)|0x300];
-	r^=*u.p++;
-	l^=((s[(r>>24)&0xff]
-	  +s[((r>>16)&0xff)|0x100])^s[((r>>8)&0xff)|0x200])+s[(r&0xff)|0x300];
-	l^=*u.p++;
-	r^=((s[(l>>24)&0xff]
-	  +s[((l>>16)&0xff)|0x100])^s[((l>>8)&0xff)|0x200])+s[(l&0xff)|0x300];
-	r^=*u.p++;
-	l^=((s[(r>>24)&0xff]
-	  +s[((r>>16)&0xff)|0x100])^s[((r>>8)&0xff)|0x200])+s[(r&0xff)|0x300];
-	l^=*u.p++;
-	r^=((s[(l>>24)&0xff]
-	  +s[((l>>16)&0xff)|0x100])^s[((l>>8)&0xff)|0x200])+s[(l&0xff)|0x300];
-	r^=*u.p++;
-	l^=((s[(r>>24)&0xff]
-	  +s[((r>>16)&0xff)|0x100])^s[((r>>8)&0xff)|0x200])+s[(r&0xff)|0x300];
-	l^=*u.p++;
-	r^=((s[(l>>24)&0xff]
-	  +s[((l>>16)&0xff)|0x100])^s[((l>>8)&0xff)|0x200])+s[(l&0xff)|0x300];
-	r^=*u.p++;
-	l^=((s[(r>>24)&0xff]
-	  +s[((r>>16)&0xff)|0x100])^s[((r>>8)&0xff)|0x200])+s[(r&0xff)|0x300];
-	l^=*u.p++;
-	r^=((s[(l>>24)&0xff]
-	  +s[((l>>16)&0xff)|0x100])^s[((l>>8)&0xff)|0x200])+s[(l&0xff)|0x300];
-	r^=*u.p++;
-	l^=((s[(r>>24)&0xff]
-	  +s[((r>>16)&0xff)|0x100])^s[((r>>8)&0xff)|0x200])+s[(r&0xff)|0x300];
-
-	/* process P17 and P18 */
-
-	l^=*u.p++;
-	r^=*u.p;
-
-	/* get fast pointer to end of data */
-
-	u.d=data;
-
-	/* return second 32 bits */
-
-	*u.d--=(WORD08)(l);
-	l>>=8;
-	*u.d--=(WORD08)(l);
-	l>>=8;
-	*u.d--=(WORD08)(l);
-	l>>=8;
-	*u.d--=(WORD08)(l);
+	l^=schedule[0];
+	t=schedule[(BITS32(l)>>24)+0x12];
+	t+=schedule[((l>>16)&0xff)+0x112];
+	t^=schedule[((l>>8)&0xff)+0x212];
+	t+=schedule[(l&0xff)+0x312];
+	r^=t;
+	r^=schedule[1];
+	t=schedule[(BITS32(r)>>24)+0x12];
+	t+=schedule[((r>>16)&0xff)+0x112];
+	t^=schedule[((r>>8)&0xff)+0x212];
+	t+=schedule[(r&0xff)+0x312];
+	l^=t;
+	l^=schedule[2];
+	t=schedule[(BITS32(l)>>24)+0x12];
+	t+=schedule[((l>>16)&0xff)+0x112];
+	t^=schedule[((l>>8)&0xff)+0x212];
+	t+=schedule[(l&0xff)+0x312];
+	r^=t;
+	r^=schedule[3];
+	t=schedule[(BITS32(r)>>24)+0x12];
+	t+=schedule[((r>>16)&0xff)+0x112];
+	t^=schedule[((r>>8)&0xff)+0x212];
+	t+=schedule[(r&0xff)+0x312];
+	l^=t;
+	l^=schedule[4];
+	t=schedule[(BITS32(l)>>24)+0x12];
+	t+=schedule[((l>>16)&0xff)+0x112];
+	t^=schedule[((l>>8)&0xff)+0x212];
+	t+=schedule[(l&0xff)+0x312];
+	r^=t;
+	r^=schedule[5];
+	t=schedule[(BITS32(r)>>24)+0x12];
+	t+=schedule[((r>>16)&0xff)+0x112];
+	t^=schedule[((r>>8)&0xff)+0x212];
+	t+=schedule[(r&0xff)+0x312];
+	l^=t;
+	l^=schedule[6];
+	t=schedule[(BITS32(l)>>24)+0x12];
+	t+=schedule[((l>>16)&0xff)+0x112];
+	t^=schedule[((l>>8)&0xff)+0x212];
+	t+=schedule[(l&0xff)+0x312];
+	r^=t;
+	r^=schedule[7];
+	t=schedule[(BITS32(r)>>24)+0x12];
+	t+=schedule[((r>>16)&0xff)+0x112];
+	t^=schedule[((r>>8)&0xff)+0x212];
+	t+=schedule[(r&0xff)+0x312];
+	l^=t;
+	l^=schedule[8];
+	t=schedule[(BITS32(l)>>24)+0x12];
+	t+=schedule[((l>>16)&0xff)+0x112];
+	t^=schedule[((l>>8)&0xff)+0x212];
+	t+=schedule[(l&0xff)+0x312];
+	r^=t;
+	r^=schedule[9];
+	t=schedule[(BITS32(r)>>24)+0x12];
+	t+=schedule[((r>>16)&0xff)+0x112];
+	t^=schedule[((r>>8)&0xff)+0x212];
+	t+=schedule[(r&0xff)+0x312];
+	l^=t;
+	l^=schedule[10];
+	t=schedule[(BITS32(l)>>24)+0x12];
+	t+=schedule[((l>>16)&0xff)+0x112];
+	t^=schedule[((l>>8)&0xff)+0x212];
+	t+=schedule[(l&0xff)+0x312];
+	r^=t;
+	r^=schedule[11];
+	t=schedule[(BITS32(r)>>24)+0x12];
+	t+=schedule[((r>>16)&0xff)+0x112];
+	t^=schedule[((r>>8)&0xff)+0x212];
+	t+=schedule[(r&0xff)+0x312];
+	l^=t;
+	l^=schedule[12];
+	t=schedule[(BITS32(l)>>24)+0x12];
+	t+=schedule[((l>>16)&0xff)+0x112];
+	t^=schedule[((l>>8)&0xff)+0x212];
+	t+=schedule[(l&0xff)+0x312];
+	r^=t;
+	r^=schedule[13];
+	t=schedule[(BITS32(r)>>24)+0x12];
+	t+=schedule[((r>>16)&0xff)+0x112];
+	t^=schedule[((r>>8)&0xff)+0x212];
+	t+=schedule[(r&0xff)+0x312];
+	l^=t;
+	l^=schedule[14];
+	t=schedule[(BITS32(l)>>24)+0x12];
+	t+=schedule[((l>>16)&0xff)+0x112];
+	t^=schedule[((l>>8)&0xff)+0x212];
+	t+=schedule[(l&0xff)+0x312];
+	r^=t;
+	r^=schedule[15];
+	t=schedule[(BITS32(r)>>24)+0x12];
+	t+=schedule[((r>>16)&0xff)+0x112];
+	t^=schedule[((r>>8)&0xff)+0x212];
+	t+=schedule[(r&0xff)+0x312];
+	l^=t;
+	l^=schedule[16];
+	r^=schedule[17];
 
 	/* return first 32 bits */
 
-	*u.d--=(WORD08)(r);
-	r>>=8;
-	*u.d--=(WORD08)(r);
-	r>>=8;
-	*u.d--=(WORD08)(r);
-	r>>=8;
-	*u.d=(WORD08)(r);
+	data[0]=(WORD08)((r)>>24);
+	data[1]=(WORD08)((r)>>16);
+	data[2]=(WORD08)((r)>>8);
+	data[3]=(WORD08) (r);
+
+	/* return second 32 bits */
+
+	data[4]=(WORD08)((l)>>24);
+	data[5]=(WORD08)((l)>>16);
+	data[6]=(WORD08)((l)>>8);
+	data[7]=(WORD08) (l);
 }
 
 /*
@@ -611,126 +656,147 @@ void CRYPTOCALL blowfish_encrypt(WORD08 *data,WORD32 *schedule)
 
 #ifndef BLOWFISH_NO_DECRYPT
 
-void CRYPTOCALL blowfish_decrypt(WORD08 *data,WORD32 *schedule)
+void CRYPTOCALL blowfish_decrypt(WORD08 *data,register WORD32 *schedule)
 {
 	register WORD32 l;			/* Xl of Blowfish	*/
 	register WORD32 r;			/* Xr of Blowfish	*/
-	register union
-	{
-		WORD08 *d;			/* 8 bit rover		*/
-		WORD32 *p;			/* 32 bit rover		*/
-	} u;					/* rover union		*/
-	register WORD32 *s=schedule+18;		/* pointer to Sm,n	*/
+	register WORD32 t;			/* work buffer		*/
 
 
-	/* get fast pointer to data */
-
-	u.d=data;
-
 	/* load first 32 bits */
 
-	l=*u.d++;
+	l=data[0];
 	l<<=8;
-	l|=*u.d++;
+	l+=data[1];
 	l<<=8;
-	l|=*u.d++;
+	l+=data[2];
 	l<<=8;
-	l|=*u.d++;
+	l|=data[3];
 
 	/* load second 32 bits */
 
-	r=*u.d++;
+	r=data[4];
 	r<<=8;
-	r|=*u.d++;
+	r+=data[5];
 	r<<=8;
-	r|=*u.d++;
+	r+=data[6];
 	r<<=8;
-	r|=*u.d;
+	r+=data[7];
 
-	/* save current data pointer, get pointer to P18 */
+	/* process P18 to P1 */
 
-	data=u.d;
-	u.p=schedule+17;
-
-	/* process P18 to P3 */
-
-	l^=*u.p--;
-	r^=((s[(l>>24)&0xff]
-	  +s[((l>>16)&0xff)|0x100])^s[((l>>8)&0xff)|0x200])+s[(l&0xff)|0x300];
-	r^=*u.p--;
-	l^=((s[(r>>24)&0xff]
-	  +s[((r>>16)&0xff)|0x100])^s[((r>>8)&0xff)|0x200])+s[(r&0xff)|0x300];
-	l^=*u.p--;
-	r^=((s[(l>>24)&0xff]
-	  +s[((l>>16)&0xff)|0x100])^s[((l>>8)&0xff)|0x200])+s[(l&0xff)|0x300];
-	r^=*u.p--;
-	l^=((s[(r>>24)&0xff]
-	  +s[((r>>16)&0xff)|0x100])^s[((r>>8)&0xff)|0x200])+s[(r&0xff)|0x300];
-	l^=*u.p--;
-	r^=((s[(l>>24)&0xff]
-	  +s[((l>>16)&0xff)|0x100])^s[((l>>8)&0xff)|0x200])+s[(l&0xff)|0x300];
-	r^=*u.p--;
-	l^=((s[(r>>24)&0xff]
-	  +s[((r>>16)&0xff)|0x100])^s[((r>>8)&0xff)|0x200])+s[(r&0xff)|0x300];
-	l^=*u.p--;
-	r^=((s[(l>>24)&0xff]
-	  +s[((l>>16)&0xff)|0x100])^s[((l>>8)&0xff)|0x200])+s[(l&0xff)|0x300];
-	r^=*u.p--;
-	l^=((s[(r>>24)&0xff]
-	  +s[((r>>16)&0xff)|0x100])^s[((r>>8)&0xff)|0x200])+s[(r&0xff)|0x300];
-	l^=*u.p--;
-	r^=((s[(l>>24)&0xff]
-	  +s[((l>>16)&0xff)|0x100])^s[((l>>8)&0xff)|0x200])+s[(l&0xff)|0x300];
-	r^=*u.p--;
-	l^=((s[(r>>24)&0xff]
-	  +s[((r>>16)&0xff)|0x100])^s[((r>>8)&0xff)|0x200])+s[(r&0xff)|0x300];
-	l^=*u.p--;
-	r^=((s[(l>>24)&0xff]
-	  +s[((l>>16)&0xff)|0x100])^s[((l>>8)&0xff)|0x200])+s[(l&0xff)|0x300];
-	r^=*u.p--;
-	l^=((s[(r>>24)&0xff]
-	  +s[((r>>16)&0xff)|0x100])^s[((r>>8)&0xff)|0x200])+s[(r&0xff)|0x300];
-	l^=*u.p--;
-	r^=((s[(l>>24)&0xff]
-	  +s[((l>>16)&0xff)|0x100])^s[((l>>8)&0xff)|0x200])+s[(l&0xff)|0x300];
-	r^=*u.p--;
-	l^=((s[(r>>24)&0xff]
-	  +s[((r>>16)&0xff)|0x100])^s[((r>>8)&0xff)|0x200])+s[(r&0xff)|0x300];
-	l^=*u.p--;
-	r^=((s[(l>>24)&0xff]
-	  +s[((l>>16)&0xff)|0x100])^s[((l>>8)&0xff)|0x200])+s[(l&0xff)|0x300];
-	r^=*u.p--;
-	l^=((s[(r>>24)&0xff]
-	  +s[((r>>16)&0xff)|0x100])^s[((r>>8)&0xff)|0x200])+s[(r&0xff)|0x300];
-
-	/* process P2 and P1 */
-
-	l^=*u.p--;
-	r^=*u.p;
-
-	/* get fast pointer to end of data */
-
-	u.d=data;
-
-	/* return second 32 bits */
-
-	*u.d--=(WORD08)(l);
-	l>>=8;
-	*u.d--=(WORD08)(l);
-	l>>=8;
-	*u.d--=(WORD08)(l);
-	l>>=8;
-	*u.d--=(WORD08)(l);
+	l^=schedule[17];
+	t=schedule[(BITS32(l)>>24)+0x12];
+	t+=schedule[((l>>16)&0xff)+0x112];
+	t^=schedule[((l>>8)&0xff)+0x212];
+	t+=schedule[(l&0xff)+0x312];
+	r^=t;
+	r^=schedule[16];
+	t=schedule[(BITS32(r)>>24)+0x12];
+	t+=schedule[((r>>16)&0xff)+0x112];
+	t^=schedule[((r>>8)&0xff)+0x212];
+	t+=schedule[(r&0xff)+0x312];
+	l^=t;
+	l^=schedule[15];
+	t=schedule[(BITS32(l)>>24)+0x12];
+	t+=schedule[((l>>16)&0xff)+0x112];
+	t^=schedule[((l>>8)&0xff)+0x212];
+	t+=schedule[(l&0xff)+0x312];
+	r^=t;
+	r^=schedule[14];
+	t=schedule[(BITS32(r)>>24)+0x12];
+	t+=schedule[((r>>16)&0xff)+0x112];
+	t^=schedule[((r>>8)&0xff)+0x212];
+	t+=schedule[(r&0xff)+0x312];
+	l^=t;
+	l^=schedule[13];
+	t=schedule[(BITS32(l)>>24)+0x12];
+	t+=schedule[((l>>16)&0xff)+0x112];
+	t^=schedule[((l>>8)&0xff)+0x212];
+	t+=schedule[(l&0xff)+0x312];
+	r^=t;
+	r^=schedule[12];
+	t=schedule[(BITS32(r)>>24)+0x12];
+	t+=schedule[((r>>16)&0xff)+0x112];
+	t^=schedule[((r>>8)&0xff)+0x212];
+	t+=schedule[(r&0xff)+0x312];
+	l^=t;
+	l^=schedule[11];
+	t=schedule[(BITS32(l)>>24)+0x12];
+	t+=schedule[((l>>16)&0xff)+0x112];
+	t^=schedule[((l>>8)&0xff)+0x212];
+	t+=schedule[(l&0xff)+0x312];
+	r^=t;
+	r^=schedule[10];
+	t=schedule[(BITS32(r)>>24)+0x12];
+	t+=schedule[((r>>16)&0xff)+0x112];
+	t^=schedule[((r>>8)&0xff)+0x212];
+	t+=schedule[(r&0xff)+0x312];
+	l^=t;
+	l^=schedule[9];
+	t=schedule[(BITS32(l)>>24)+0x12];
+	t+=schedule[((l>>16)&0xff)+0x112];
+	t^=schedule[((l>>8)&0xff)+0x212];
+	t+=schedule[(l&0xff)+0x312];
+	r^=t;
+	r^=schedule[8];
+	t=schedule[(BITS32(r)>>24)+0x12];
+	t+=schedule[((r>>16)&0xff)+0x112];
+	t^=schedule[((r>>8)&0xff)+0x212];
+	t+=schedule[(r&0xff)+0x312];
+	l^=t;
+	l^=schedule[7];
+	t=schedule[(BITS32(l)>>24)+0x12];
+	t+=schedule[((l>>16)&0xff)+0x112];
+	t^=schedule[((l>>8)&0xff)+0x212];
+	t+=schedule[(l&0xff)+0x312];
+	r^=t;
+	r^=schedule[6];
+	t=schedule[(BITS32(r)>>24)+0x12];
+	t+=schedule[((r>>16)&0xff)+0x112];
+	t^=schedule[((r>>8)&0xff)+0x212];
+	t+=schedule[(r&0xff)+0x312];
+	l^=t;
+	l^=schedule[5];
+	t=schedule[(BITS32(l)>>24)+0x12];
+	t+=schedule[((l>>16)&0xff)+0x112];
+	t^=schedule[((l>>8)&0xff)+0x212];
+	t+=schedule[(l&0xff)+0x312];
+	r^=t;
+	r^=schedule[4];
+	t=schedule[(BITS32(r)>>24)+0x12];
+	t+=schedule[((r>>16)&0xff)+0x112];
+	t^=schedule[((r>>8)&0xff)+0x212];
+	t+=schedule[(r&0xff)+0x312];
+	l^=t;
+	l^=schedule[3];
+	t=schedule[(BITS32(l)>>24)+0x12];
+	t+=schedule[((l>>16)&0xff)+0x112];
+	t^=schedule[((l>>8)&0xff)+0x212];
+	t+=schedule[(l&0xff)+0x312];
+	r^=t;
+	r^=schedule[2];
+	t=schedule[(BITS32(r)>>24)+0x12];
+	t+=schedule[((r>>16)&0xff)+0x112];
+	t^=schedule[((r>>8)&0xff)+0x212];
+	t+=schedule[(r&0xff)+0x312];
+	l^=t;
+	l^=schedule[1];
+	r^=schedule[0];
 
 	/* return first 32 bits */
 
-	*u.d--=(WORD08)(r);
-	r>>=8;
-	*u.d--=(WORD08)(r);
-	r>>=8;
-	*u.d--=(WORD08)(r);
-	r>>=8;
-	*u.d=(WORD08)(r);
+	data[0]=(WORD08)((r)>>24);
+	data[1]=(WORD08)((r)>>16);
+	data[2]=(WORD08)((r)>>8);
+	data[3]=(WORD08) (r);
+
+	/* return second 32 bits */
+
+	data[4]=(WORD08)((l)>>24);
+	data[5]=(WORD08)((l)>>16);
+	data[6]=(WORD08)((l)>>8);
+	data[7]=(WORD08) (l);
 }
 
 #endif
