@@ -1,8 +1,9 @@
 /*
  * virtual private network daemon (vpnd)
  *
- * cryptographic stuff (c) 1999 Andreas Steinmetz, astmail@yahoo.com
+ * cryptographic stuff (c) 1999 Andreas Steinmetz, ast@domdv.de
  * other code (c) 1999 D.O.M. Datenverarbeitung GmbH, author Andreas Steinmetz
+ * (c) 2001,2005 Andreas Steinmetz
  *
  * License:
  * This code is in the public domain (*) under the GNU public license.
@@ -29,6 +30,8 @@ WORD08 verbose=0;			/* verbose mode for script test	*/
 WORD08 terminate=0;			/* termination request (signal)	*/
 WORD08 dns=0;				/* dns lookup allowed flag	*/
 WORD32 facility=LOG_DAEMON;		/* syslog facility		*/
+char *control=NULL;			/* control socket pathname	*/
+int ctrl=-1;				/* control socket		*/
 
 #ifdef DEBUG
 int debug=0;				/* debug level			*/
@@ -70,11 +73,22 @@ void initvars(VPN *anchor)
 		anchor->serial=anchor->lock=-1;
 	anchor->speed=B115200;
 	anchor->conffile="/etc/vpnd.conf";
+#if defined(OSOpenBSD)
+	anchor->randomdev="/dev/srandom";
+#elif defined(SunOS)
+	anchor->randomdev="/var/run/egd-pool";
+#else
 	anchor->randomdev="/dev/random";
+#endif
 	anchor->keyfile="/etc/vpnd.key";
 	anchor->lkeyfile="vpnd.lcl.key";
 	anchor->rkeyfile="vpnd.rmt.key";
 	anchor->cipher=BLOWFISH;
 	anchor->encrypt=blowfish_encrypt;
 	memcpy(anchor->piv,dftiv,BLOWFISH_BLOCKSIZE);
+	anchor->ifhandler=sliphandler;
+	anchor->iftype="slip";
+	anchor->pkt_init=pkg_slip_init;
+	anchor->if_to_pkt=slip_to_pkg;
+	anchor->pkt_to_if=pkg_to_slip;
 }
