@@ -1,8 +1,9 @@
 /*
  * virtual private network daemon (vpnd)
  *
- * cryptographic stuff (c) 1999 Andreas Steinmetz, astmail@yahoo.com
+ * cryptographic stuff (c) 1999 Andreas Steinmetz, ast@domdv.de
  * other code (c) 1999 D.O.M. Datenverarbeitung GmbH, author Andreas Steinmetz
+ * (c) 2001 Andreas Steinmetz
  *
  * License:
  * This code is in the public domain (*) under the GNU public license.
@@ -63,9 +64,11 @@ WORD32 getaddr(WORD08 *ip)
 
 		for(i=0;i<4;*ptr++='.',i++)
 		{
-			if(h->h_addr[i]>99)*ptr++=h->h_addr[i]/100+'0';
-			if(h->h_addr[i]>9)*ptr++=(h->h_addr[i]%100)/10+'0';
-			*ptr++=h->h_addr[i]%10+'0';
+			if((WORD08)(h->h_addr[i])>99)*ptr++=
+				(WORD08)(h->h_addr[i])/100+'0';
+			if((WORD08)(h->h_addr[i])>9)*ptr++=
+				((WORD08)(h->h_addr[i])%100)/10+'0';
+			*ptr++=(WORD08)(h->h_addr[i])%10+'0';
 		}
 		*(--ptr)=0;
 
@@ -89,6 +92,8 @@ WORD32 getaddr(WORD08 *ip)
  *	   2 - bind failed
  *	   3 - set socket option or socket ioctl failed
  *	   4 - connect/listen failed
+ *	   5 - socks protocol error
+ *	   6 - socks server connection failure/reject
  *
  * This procedure establishes a tcp socket in client or server mode.
  */
@@ -103,6 +108,8 @@ int tcpsocket(VPN *anchor)
 	struct linger l;	/* delay on closing socket	*/
 	fd_set w;		/* used to wait for connect	*/
 	struct timeval tv;	/* used to expire connect	*/
+	WORD32 target;		/* server ip for socks connect	*/
+	WORD08 socksmsg[9];	/* socks v4 protocol buffer	*/
 
 
 	/* debug message */
@@ -126,6 +133,7 @@ int tcpsocket(VPN *anchor)
 		x=1;
 		if(setsockopt(s,SOL_SOCKET,SO_REUSEADDR,&x,sizeof(x)))
 			JUMP("setsockopt(SO_REUSEADDR)",err2);
+		memset(&a,0,sizeof(a));
 		a.sin_family=AF_INET;
 		a.sin_addr.s_addr=
 		    getaddr(anchor->mode?anchor->client_ip:anchor->server_ip);
@@ -149,8 +157,13 @@ int tcpsocket(VPN *anchor)
 	/* prevent packet coalescence */
 
 	x=1;
+#ifdef SOL_TCP
 	if(setsockopt(s,SOL_TCP,TCP_NODELAY,&x,sizeof(x)))
 		JUMP("setsockopt(TCP_NODELAY)",err3);
+#elif IPPROTO_TCP
+	if(setsockopt(s,IPPROTO_TCP,TCP_NODELAY,&x,sizeof(x)))
+		JUMP("setsockopt(TCP_NODELAY)",err3);
+#endif
 
 	/* set type of service and precedence, if given
 	   (if any router actually uses these the precedence
@@ -162,24 +175,41 @@ int tcpsocket(VPN *anchor)
 	if(anchor->ipopts)
 	{
 		x=((anchor->ipopts)<<2)&0x3c;
+#ifdef SOL_IP
 		if(setsockopt(s,SOL_IP,IP_TOS,&x,sizeof(x)))
 			JUMP("setsockopt(IP_TOS)",err3);
+#elif IPPROTO_IP
+		if(setsockopt(s,IPPROTO_IP,IP_TOS,&x,sizeof(x)))
+			JUMP("setsockopt(IP_TOS)",err3);
+#endif
 	}
 
 	/* set socket to nonblocking, handle errors */
 
+#ifdef SunOS
+	if(fcntl(s,F_SETFL,O_NONBLOCK))JUMP("fcntl(O_NONBLOCK)",err3);
+#else
 	x=1;
 	if(ioctl(s,FIONBIO,&x))JUMP("ioctl(FIONBIO)",err3);
+#endif
 
 	/* client mode */
 
 	if(anchor->mode)
 	{
-		/* connect to peer, handle errors */
+		/* connect to peer or socks server, handle errors */
 
 		a.sin_family=AF_INET;
-		a.sin_addr.s_addr=getaddr(anchor->server_ip);
-		a.sin_port=htons(anchor->server_port);
+		if(anchor->socksip)
+		{
+			a.sin_addr.s_addr=getaddr(anchor->socksip);
+			a.sin_port=htons(anchor->socksport);
+		}
+		else
+		{
+			a.sin_addr.s_addr=getaddr(anchor->server_ip);
+			a.sin_port=htons(anchor->server_port);
+		}
 
 		if(connect(s,(struct sockaddr *)(&a),sizeof(a)))
 		{
@@ -224,6 +254,46 @@ int tcpsocket(VPN *anchor)
 		/* make peer handle globally available */
 
 		anchor->peer=s;
+
+		/* in case of connect to socks server */
+
+		if(anchor->socksip)
+		{
+			/* prepare socks protocol message */
+
+			target=ntohl(getaddr(anchor->server_ip));
+			socksmsg[0]=4;
+			socksmsg[1]=1;
+			socksmsg[2]=(WORD08)((anchor->server_port)>>8);
+			socksmsg[3]=(WORD08)(anchor->server_port);
+			socksmsg[4]=(WORD08)(target>>24);
+			socksmsg[5]=(WORD08)(target>>16);
+			socksmsg[6]=(WORD08)(target>>8);
+			socksmsg[7]=(WORD08)(target);
+			socksmsg[8]=0;
+
+			/* send socks request, handle errors */
+
+			if(anchor->socksuser)
+			{
+				if(!dosend(socksmsg,8,anchor))
+					GOTO("socks request",err5);
+				if(!dosend(anchor->socksuser,
+				    strlen(anchor->socksuser),anchor))
+					GOTO("socks request",err5);
+				if(!dosend(socksmsg+8,1,anchor))
+					GOTO("socks request",err5);
+			}
+			else if(!dosend(socksmsg,9,anchor))
+				GOTO("socks request",err5);
+
+			/* read and process socks reply, handle errors */
+
+			if(!dorecv(socksmsg,8,anchor))
+				GOTO("socks reply",err5);
+			if(socksmsg[0]!=0||socksmsg[1]!=90)
+				GOTO("socks connect failure",err6);
+		}
 	}
 
 	/* server mode */
@@ -247,6 +317,10 @@ int tcpsocket(VPN *anchor)
 
 	goto err0;
 
+err6:	err++;
+err5:	err++;
+	shutdown(anchor->peer,2);
+	anchor->peer=-1;
 err4:	err++;
 err3:	err++;
 err2:	err++;
@@ -325,6 +399,7 @@ err0:	return err;
  * input: dmode  - 0 for peer socket
  *	         - 1 for server socket
  *	         - 2 for peer and server socket
+ *		 - 4 forced close of tcp peer socket
  *	  anchor - pointer to vpnd global data
  *
  *
@@ -334,6 +409,9 @@ err0:	return err;
 
 void disconnect(int dmode,VPN *anchor)
 {
+	struct linger l;	/* delay on closing socket */
+
+
 	/* debug message */
 
 	ENTER("disconnect");
@@ -346,9 +424,20 @@ void disconnect(int dmode,VPN *anchor)
 
 		if(!anchor->serialmode)
 		{
-			/* send TCP_FIN */
+			/* disable linger on close time  if forced close */
 
-			if(shutdown(anchor->peer,2))ERRNO("shutdown");
+			if(dmode&4)
+			{
+				l.l_onoff=0;
+				l.l_linger=0;
+				if(setsockopt(anchor->peer,SOL_SOCKET,
+					SO_LINGER,&l,sizeof(l)))
+						ERRNO("setsockopt(SO_LINGER)");
+			}
+
+			/* send TCP_FIN in case of regular close */
+
+			else if(shutdown(anchor->peer,2))ERRNO("shutdown");
 
 			/* close communication channel */
 
@@ -370,7 +459,7 @@ void disconnect(int dmode,VPN *anchor)
 
 	/* if server socket and if the server socket exists */
 
-	if(dmode)if(anchor->server!=-1)
+	if(dmode&3)if(anchor->server!=-1)
 	{
 		/* close server socket */
 
